SystemOrganization addCategory: #'Redis-Queue'!
SystemOrganization addCategory: #'Redis-Queue-Tests'!

TestCase subclass: #RedisQueueTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Redis-Queue-Tests'!

!RedisQueueTests methodsFor: 'testing' stamp: 'mlh 1/17/2010 20:02'!
test
	| worker job sourceConnection semaphore |
	RedisMockTarget clearCount: (semaphore := Semaphore new).
	worker := RedisWorker connection: RedisConnection localhost queues: {'high'. 'medium'}.
	worker log: Transcript.
	worker start.
	job := RedisJob
				class: RedisMockTarget
				selector: #count:
				args: {'Success'}.
	sourceConnection := RedisConnection localhost.
	10
		timesRepeat: [job enqueue: 'high' on: sourceConnection].
	10
		timesRepeat: [semaphore wait].
	self assert: RedisMockTarget count = 10.
	worker stopNow.
	sourceConnection flushDb; quit! !

Object subclass: #RedisExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Redis-Queue'!

!RedisExample class methodsFor: 'redis-jobs' stamp: 'mlh 1/18/2010 21:36'!
fail: errorString 
	Transcript show: 'Fail Job Received';
		 show: String crlf.
	RedisError signal: errorString! !

!RedisExample class methodsFor: 'redis-jobs' stamp: 'mlh 1/18/2010 21:36'!
show: aString 
	Transcript show: 'Show Job Finished: ' , aString;
		 show: String crlf! !

!RedisExample class methodsFor: 'redis-jobs' stamp: 'mlh 1/18/2010 21:34'!
wait: aString 
	(Delay forSeconds: aString asNumber) wait.
	Transcript show: 'Wait Job Finished' ; show: String crlf! !

Object subclass: #RedisJob
	instanceVariableNames: 'class args selector id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Redis-Queue'!

!RedisJob class methodsFor: 'instance creation' stamp: 'mlh 1/17/2010 12:13'!
class: aClass selector: aSelector args: args 
	^ self new class: aClass name;
		 selector: aSelector;
		 args: args;
		 yourself! !

!RedisJob class methodsFor: 'instance creation' stamp: 'mlh 1/16/2010 14:21'!
readFrom: aStream 
	^ self new readFrom: aStream! !

!RedisJob methodsFor: 'accessing' stamp: 'mlh 1/17/2010 12:10'!
args: anObject
	args := anObject! !

!RedisJob methodsFor: 'accessing' stamp: 'mlh 1/17/2010 12:09'!
class: anObject
	class := anObject! !

!RedisJob methodsFor: 'api-queue' stamp: 'mlh 1/18/2010 15:55'!
enqueue: aQueue on: connection 
	connection sAdd: 'queues' member: aQueue.
	connection
		lPush: aQueue
		value: (Json render: self)! !

!RedisJob methodsFor: 'private' stamp: 'mlh 1/17/2010 17:53'!
execute
	class perform: selector withArguments: args! !

!RedisJob methodsFor: 'private' stamp: 'mlh 1/17/2010 12:03'!
jsonWriteOn: aStream 
	| payload |
	payload := JsonObject new.
	payload class: class.
	payload selector: selector.
	payload args: args.
	payload
		id: (SecureHashAlgorithm new
				hashMessage: (Json render: payload)).
	payload jsonWriteOn: aStream! !

!RedisJob methodsFor: 'parsing' stamp: 'mlh 1/17/2010 12:24'!
readFrom: aStream
	| json |
	json := Json readFrom: aStream.
	class := Smalltalk at: ( json at: 'class' ) asSymbol.
	selector := ( json at: 'selector' ) asSymbol.
	args := json at: 'args'.
	id := json at: 'id'.
	^self! !

!RedisJob methodsFor: 'accessing' stamp: 'mlh 1/17/2010 12:09'!
selector: anObject
	selector := anObject! !

Object subclass: #RedisMockTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Redis-Queue-Tests'!

RedisMockTarget class
	instanceVariableNames: 'count semaphore'!

RedisMockTarget class
	instanceVariableNames: 'count semaphore'!

!RedisMockTarget class methodsFor: 'testing-support' stamp: 'mlh 1/17/2010 20:00'!
clearCount: aSemaphore
	semaphore := aSemaphore.
	count := 0! !

!RedisMockTarget class methodsFor: 'testing-support' stamp: 'mlh 1/17/2010 19:50'!
count
	^ count ! !

!RedisMockTarget class methodsFor: 'testing-support' stamp: 'mlh 1/17/2010 19:59'!
count: args 
	count := count + 1.
	semaphore signal.! !

!RedisMockTarget class methodsFor: 'class initialization' stamp: 'mlh 1/17/2010 19:49'!
initialize
	count := 0! !

Object subclass: #RedisMonitor
	instanceVariableNames: 'connection process log continue queues workers started finished failed mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Redis-Queue'!

!RedisMonitor class methodsFor: 'instance creation' stamp: 'mlh 1/18/2010 15:59'!
connection: aConnection 
	^ self new connection: aConnection! !

!RedisMonitor methodsFor: 'accessing' stamp: 'mlh 1/18/2010 16:00'!
connection
	^ connection! !

!RedisMonitor methodsFor: 'accessing' stamp: 'mlh 1/18/2010 16:00'!
connection: anObject
	connection := anObject! !

!RedisMonitor methodsFor: 'accessing' stamp: 'mlh 1/18/2010 16:04'!
continue
	^ continue! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/28/2010 22:15'!
currentJob: aWorkerId
	^ connection get: 'worker:job:' , aWorkerId! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/28/2010 22:14'!
currentQueue: aWorkerId
	^ connection get: 'worker:queue:' , aWorkerId! !

!RedisMonitor methodsFor: 'accessing' stamp: 'MikeHales 1/24/2010 20:42'!
failed
	^ failed ifNil: [0]! !

!RedisMonitor methodsFor: 'accessing' stamp: 'MikeHales 1/24/2010 20:42'!
finished
	^ finished ifNil: [0]! !

!RedisMonitor methodsFor: 'initialize-release' stamp: 'mlh 1/18/2010 16:17'!
initialize
	continue := true.
	mutex := Semaphore forMutualExclusion! !

!RedisMonitor methodsFor: 'testing' stamp: 'MikeHales 1/24/2010 20:53'!
isRunning
	^connection isConnected and: [ process notNil and: [ process isTerminated not ] ]! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/24/2010 20:37'!
jobsFailed: aQueue
	^ connection get: 'stats:failed:' , aQueue! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/24/2010 20:37'!
jobsFinished: aQueue
	^ connection get: 'stats:finished:' , aQueue! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/24/2010 20:36'!
jobsStarted: aQueue
	^ connection get: 'stats:started:' , aQueue! !

!RedisMonitor methodsFor: 'accessing' stamp: 'mlh 1/18/2010 21:08'!
log
	^ log! !

!RedisMonitor methodsFor: 'accessing' stamp: 'mlh 1/18/2010 21:08'!
log: aStream
	log := aStream! !

!RedisMonitor methodsFor: 'private-monitor' stamp: 'MikeHales 1/24/2010 21:00'!
monitor
	mutex
		critical: [
			queues := connection sMembers: 'queues'.
			workers := connection sMembers: 'workers'.
			started := connection get: 'stats:started'.
			finished := connection get: 'stats:finished'.
			failed := connection get: 'stats:failed']! !

!RedisMonitor methodsFor: 'private-monitor' stamp: 'mlh 1/18/2010 16:54'!
monitorLoop
	self report: 'Entering Monitor Loop'.
	process := [[self continue]
				whileTrue: [self monitor.
					(Delay forSeconds: 5) wait].
			self stopNow]
				forkAt: self monitorPriority! !

!RedisMonitor methodsFor: 'private-monitor' stamp: 'mlh 1/18/2010 16:10'!
monitorPriority
	^Processor userBackgroundPriority! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/24/2010 20:36'!
queueSize: aQueue
	^ connection lLen: aQueue! !

!RedisMonitor methodsFor: 'accessing' stamp: 'MikeHales 1/24/2010 20:53'!
queues
	^ queues ifNil: [#()]! !

!RedisMonitor methodsFor: 'private-monitor' stamp: 'mlh 1/18/2010 21:07'!
report: aString 
	log
		ifNotNil: [log nextPutAll: aString;
				 nextPutAll: String crlf;
				 flush]! !

!RedisMonitor methodsFor: 'commands' stamp: 'MikeHales 1/24/2010 20:54'!
start
	continue := true.
	self report: 'Starting'.
	self monitorLoop! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/28/2010 22:15'!
startTime: aWorkerId
	^ connection get: 'worker:started:' , aWorkerId! !

!RedisMonitor methodsFor: 'accessing' stamp: 'MikeHales 1/24/2010 20:42'!
started
	^ started ifNil: [0]! !

!RedisMonitor methodsFor: 'queries' stamp: 'mlh 1/18/2010 21:44'!
stats
	| stats |
	stats := Dictionary new.
	stats at: 'Queues' put: queues;
		 at: 'Workers' put: workers;
		 at: 'Jobs Started' put: started;
		 at: 'Jobs Finished' put: finished;
		 at: 'Jobs Failed' put: failed.
	^stats! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/24/2010 20:37'!
statsForQueue: aQueue
	mutex
		critical:
			[ | stats |
			stats := Dictionary new.
			stats
				at: 'Queue Length' put: (self queueSize: aQueue);
				at: 'Jobs Started' put: (self jobsStarted: aQueue);
				at: 'Jobs Finished' put: (self jobsFinished: aQueue);
				at: 'Jobs Failed' put: (self jobsFailed: aQueue);
				at: 'Current Workers' put: (self workersOn: aQueue).
			^ stats ]! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/28/2010 22:17'!
statsForWorker: aWorkerId
	mutex
		critical:
			[ | stats stream |
			stats := Dictionary new.
			stream := aWorkerId readStream.
			stats
				at: 'Hostname' put: (stream upTo: $:);
				at: 'Redis Server' put: (stream upTo: $:);
				at: 'Queues' put: stream upToEnd;
				at: 'Current Queue' put: (self currentQueue: aWorkerId);
				at: 'Current Job' put: (self currentJob: aWorkerId);
				at: 'Start Time' put: (self startTime: aWorkerId);
				at: 'Jobs Started' put: (self jobsStarted: aWorkerId);
				at: 'Jobs Finished' put: (self jobsFinished: aWorkerId);
				at: 'Jobs Failed' put: (self jobsFailed: aWorkerId).
			^ stats ]! !

!RedisMonitor methodsFor: 'commands' stamp: 'mlh 1/18/2010 16:05'!
stop
	continue := false! !

!RedisMonitor methodsFor: 'commands' stamp: 'mlh 1/18/2010 16:07'!
stopNow
	self report: 'Terminating Process, Closing Connection'.
	process terminate.
	connection quit.! !

!RedisMonitor methodsFor: 'accessing' stamp: 'MikeHales 1/24/2010 20:42'!
workers
	^ workers ifNil: [#()]! !

!RedisMonitor methodsFor: 'queries' stamp: 'MikeHales 1/24/2010 20:37'!
workersOn: aQueue
	^ connection sMembers: 'queue:working:' , aQueue! !

Object subclass: #RedisWorker
	instanceVariableNames: 'connection queues process log continue job id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Redis-Queue'!

!RedisWorker class methodsFor: 'instance creation' stamp: 'mlh 1/14/2010 22:12'!
connection: aRedisConnection queues: queues 
	^ self new connection: aRedisConnection;
		 queues: queues;
		 yourself! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/14/2010 22:23'!
clearStat: aStatName
	^ connection rem: 'stats:' , aStatName! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/14/2010 22:13'!
connection
	^ connection! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/14/2010 22:13'!
connection: anObject
	connection := anObject! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/18/2010 16:05'!
continue
	^ continue! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/14/2010 22:23'!
decrStat: aStatName
	^ connection decr: 'stats:' , aStatName! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/14/2010 22:22'!
getStat: aStatName
	^ connection get: 'stats:' , aStatName! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/28/2010 22:30'!
id
	^ id
		ifNil: [id := NetNameResolver localHostName , '-' , ThisOSProcess thisOSProcess pid printString]! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/14/2010 22:22'!
incrStat: aStatName
	^ connection incr: 'stats:' , aStatName! !

!RedisWorker methodsFor: 'initialize-release' stamp: 'mlh 1/14/2010 22:18'!
initialize
	continue := true! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/28/2010 22:46'!
jobFailed
	connection del: { 'worker:queue:' , self id }.
	connection del: { 'worker:job:', self id }.
	connection sRem: 'queue:working:' , job first member: self id.
	self incrStat: 'failed'.
	self incrStat: 'failed:' , self id.
	self incrStat: 'failed:' , job first.
	self report: 'Job Failed'! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/28/2010 22:46'!
jobFinished

	connection del: { 'worker:queue:' , self id }.
	connection del: { 'worker:job:', self id }.
	connection sRem: 'queue:working:' , job first member: self id.
	self incrStat: 'finished'.
	self incrStat: 'finished:' , self id.
	self incrStat: 'finished:' , job first.
	self report: 'Finished Job'.
	
! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/24/2010 13:19'!
jobStarted
	connection set: 'worker:queue:' , self id value: job first.
	connection set: 'worker:job:', self id value: job second.
	connection sAdd: 'queue:working:' , job first member: self id.
	self incrStat: 'started'.
	self incrStat: 'started:' , self id.
	self incrStat: 'started:' , job first.
	self report: 'Started Job'! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/14/2010 22:32'!
log
	^ log! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/14/2010 22:31'!
log: aStream
	log := aStream! !

!RedisWorker methodsFor: 'private-work' stamp: 'mlh 1/18/2010 21:30'!
performJob: jsonString 

	[(RedisJob readFrom: jsonString readStream) execute]
		on: Error
		do: [:ex | self jobFailed]! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/14/2010 22:13'!
queues
	^ queues! !

!RedisWorker methodsFor: 'accessing' stamp: 'mlh 1/14/2010 22:13'!
queues: anObject
	queues := anObject! !

!RedisWorker methodsFor: 'private-work' stamp: 'mlh 1/18/2010 15:34'!
report: aString 
	log
		ifNotNil: [log nextPutAll: aString;
				 nextPutAll: String crlf;
				 flush]! !

!RedisWorker methodsFor: 'commands' stamp: 'mlh 1/17/2010 20:24'!
start
	self workerStarted.
	self workLoop! !

!RedisWorker methodsFor: 'commands' stamp: 'mlh 1/17/2010 20:35'!
stop
	self report: 'Stopping:' , self id.
	continue := false.
	job
		ifNil: [self stopNow]! !

!RedisWorker methodsFor: 'commands' stamp: 'mlh 1/17/2010 20:35'!
stopNow
	self workerStopped.
	self report: 'Terminating Process, Closing Connection'.
	process terminate.
	connection quit! !

!RedisWorker methodsFor: 'private-work' stamp: 'mlh 1/18/2010 21:21'!
work
	job := connection brPop: queues timeout: 30.
	job
		ifNotNil: [self jobStarted.
			self performJob: job second.
			self jobFinished].
	job := nil! !

!RedisWorker methodsFor: 'private-work' stamp: 'mlh 1/17/2010 18:29'!
workLoop
	self report: 'Entering Work Loop'.
	process := [[self continue]
				whileTrue: [self work].
			self stopNow]
				forkAt: self workPriority! !

!RedisWorker methodsFor: 'private-work' stamp: 'mlh 1/17/2010 20:15'!
workPriority
	^ Processor userBackgroundPriority! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/18/2010 20:54'!
workerStarted
	connection set: 'worker:started:' , self id value: DateAndTime now printString.
	connection sAdd: 'workers' member: self id.
	queues
		do: [:ea | connection sAdd: 'queue:listening:' , ea member: self id].
	self report: 'Starting:' , self id! !

!RedisWorker methodsFor: 'stats' stamp: 'mlh 1/28/2010 22:47'!
workerStopped
	connection del: { 'worker:started:' , self id }.
	connection sRem: 'workers' member: self id.
	queues
		do: [:ea | connection sRem: 'queue:listening:' , ea member: self id].
	self report: 'Stopping:' , self id! !
RedisMockTarget initialize!
